package org.dromara.game.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.RequiredArgsConstructor;
import org.dromara.common.core.utils.MapstructUtils;
import org.dromara.common.core.utils.StringUtils;
import org.dromara.common.mybatis.core.page.PageQuery;
import org.dromara.common.mybatis.core.page.TableDataInfo;
import org.dromara.game.domain.PlatProductCategory;
import org.dromara.game.domain.bo.PlatProductCategoryBo;
import org.dromara.game.domain.vo.PlatProductCategoryVo;
import org.dromara.game.mapper.PlatProductCategoryMapper;
import org.dromara.game.service.IPlatProductCategoryService;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 产品分类Service业务层处理
 *
 * @author Lion Li
 * @date 2024-08-07
 */
@RequiredArgsConstructor
@Service
public class PlatProductCategoryServiceImpl implements IPlatProductCategoryService {

    private final PlatProductCategoryMapper baseMapper;

    /**
     * 查询产品分类
     *
     * @param id 主键
     * @return 产品分类
     */
    @Override
    public PlatProductCategoryVo queryById(Long id){
        return baseMapper.selectVoById(id);
    }

    /**
     * 分页查询产品分类列表
     *
     * @param bo        查询条件
     * @param pageQuery 分页参数
     * @return 产品分类分页列表
     */
    @Override
    public TableDataInfo<PlatProductCategoryVo> queryPageList(PlatProductCategoryBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<PlatProductCategory> lqw = buildQueryWrapper(bo);
        Page<PlatProductCategoryVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询符合条件的产品分类列表
     *
     * @param bo 查询条件
     * @return 产品分类列表
     */
    @Override
    public List<PlatProductCategoryVo> queryList(PlatProductCategoryBo bo) {
        LambdaQueryWrapper<PlatProductCategory> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<PlatProductCategory> buildQueryWrapper(PlatProductCategoryBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<PlatProductCategory> lqw = Wrappers.lambdaQuery();
        lqw.like(StringUtils.isNotBlank(bo.getName()), PlatProductCategory::getName, bo.getName());
        return lqw;
    }

    /**
     * 新增产品分类
     *
     * @param bo 产品分类
     * @return 是否新增成功
     */
    @Override
    public Boolean insertByBo(PlatProductCategoryBo bo) {
        PlatProductCategory add = MapstructUtils.convert(bo, PlatProductCategory.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
        }
        return flag;
    }

    /**
     * 修改产品分类
     *
     * @param bo 产品分类
     * @return 是否修改成功
     */
    @Override
    public Boolean updateByBo(PlatProductCategoryBo bo) {
        PlatProductCategory update = MapstructUtils.convert(bo, PlatProductCategory.class);
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(PlatProductCategory entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 校验并批量删除产品分类信息
     *
     * @param ids     待删除的主键集合
     * @param isValid 是否进行有效性校验
     * @return 是否删除成功
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteByIds(ids) > 0;
    }
}
