package org.dromara.game.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.RequiredArgsConstructor;
import org.dromara.common.core.utils.MapstructUtils;
import org.dromara.common.core.utils.StringUtils;
import org.dromara.common.mybatis.core.page.PageQuery;
import org.dromara.common.mybatis.core.page.TableDataInfo;
import org.dromara.game.domain.PlatProductProxy;
import org.dromara.game.domain.bo.PlatProductProxyBo;
import org.dromara.game.domain.vo.PlatProductProxyVo;
import org.dromara.game.mapper.PlatProductProxyMapper;
import org.dromara.game.service.IPlatProductProxyService;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 产品代理商Service业务层处理
 *
 * @author Hixon
 * @date 2024-08-07
 */
@RequiredArgsConstructor
@Service
public class PlatProductProxyServiceImpl implements IPlatProductProxyService {

    private final PlatProductProxyMapper baseMapper;

    /**
     * 查询产品代理商
     *
     * @param id 主键
     * @return 产品代理商
     */
    @Override
    public PlatProductProxyVo queryById(Long id){
        return baseMapper.selectVoById(id);
    }

    /**
     * 分页查询产品代理商列表
     *
     * @param bo        查询条件
     * @param pageQuery 分页参数
     * @return 产品代理商分页列表
     */
    @Override
    public TableDataInfo<PlatProductProxyVo> queryPageList(PlatProductProxyBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<PlatProductProxy> lqw = buildQueryWrapper(bo);
        Page<PlatProductProxyVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询符合条件的产品代理商列表
     *
     * @param bo 查询条件
     * @return 产品代理商列表
     */
    @Override
    public List<PlatProductProxyVo> queryList(PlatProductProxyBo bo) {
        LambdaQueryWrapper<PlatProductProxy> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<PlatProductProxy> buildQueryWrapper(PlatProductProxyBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<PlatProductProxy> lqw = Wrappers.lambdaQuery();
        lqw.like(StringUtils.isNotBlank(bo.getName()), PlatProductProxy::getName, bo.getName());
        return lqw;
    }

    /**
     * 新增产品代理商
     *
     * @param bo 产品代理商
     * @return 是否新增成功
     */
    @Override
    public Boolean insertByBo(PlatProductProxyBo bo) {
        PlatProductProxy add = MapstructUtils.convert(bo, PlatProductProxy.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
        }
        return flag;
    }

    /**
     * 修改产品代理商
     *
     * @param bo 产品代理商
     * @return 是否修改成功
     */
    @Override
    public Boolean updateByBo(PlatProductProxyBo bo) {
        PlatProductProxy update = MapstructUtils.convert(bo, PlatProductProxy.class);
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(PlatProductProxy entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 校验并批量删除产品代理商信息
     *
     * @param ids     待删除的主键集合
     * @param isValid 是否进行有效性校验
     * @return 是否删除成功
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteByIds(ids) > 0;
    }
}
