package org.dromara.game.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.RequiredArgsConstructor;
import org.dromara.common.core.utils.MapstructUtils;
import org.dromara.common.core.utils.StringUtils;
import org.dromara.common.mybatis.core.page.PageQuery;
import org.dromara.common.mybatis.core.page.TableDataInfo;
import org.dromara.game.domain.PlatProduct;
import org.dromara.game.domain.bo.PlatProductBo;
import org.dromara.game.domain.vo.PlatProductVo;
import org.dromara.game.mapper.PlatProductMapper;
import org.dromara.game.service.IPlatProductService;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 产品Service业务层处理
 *
 * @author Hixon
 * @date 2024-08-07
 */
@RequiredArgsConstructor
@Service
public class PlatProductServiceImpl implements IPlatProductService {

    private final PlatProductMapper baseMapper;

    /**
     * 查询产品
     *
     * @param id 主键
     * @return 产品
     */
    @Override
    public PlatProductVo queryById(Long id){
        return baseMapper.selectVoById(id);
    }

    /**
     * 分页查询产品列表
     *
     * @param bo        查询条件
     * @param pageQuery 分页参数
     * @return 产品分页列表
     */
    @Override
    public TableDataInfo<PlatProductVo> queryPageList(PlatProductBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<PlatProduct> lqw = buildQueryWrapper(bo);
        Page<PlatProductVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询符合条件的产品列表
     *
     * @param bo 查询条件
     * @return 产品列表
     */
    @Override
    public List<PlatProductVo> queryList(PlatProductBo bo) {
        LambdaQueryWrapper<PlatProduct> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<PlatProduct> buildQueryWrapper(PlatProductBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<PlatProduct> lqw = Wrappers.lambdaQuery();
        lqw.eq(bo.getCategoryId() != null, PlatProduct::getCategoryId, bo.getCategoryId());
        lqw.like(StringUtils.isNotBlank(bo.getName()), PlatProduct::getName, bo.getName());
        return lqw;
    }

    /**
     * 新增产品
     *
     * @param bo 产品
     * @return 是否新增成功
     */
    @Override
    public Boolean insertByBo(PlatProductBo bo) {
        PlatProduct add = MapstructUtils.convert(bo, PlatProduct.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
        }
        return flag;
    }

    /**
     * 修改产品
     *
     * @param bo 产品
     * @return 是否修改成功
     */
    @Override
    public Boolean updateByBo(PlatProductBo bo) {
        PlatProduct update = MapstructUtils.convert(bo, PlatProduct.class);
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(PlatProduct entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 校验并批量删除产品信息
     *
     * @param ids     待删除的主键集合
     * @param isValid 是否进行有效性校验
     * @return 是否删除成功
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteByIds(ids) > 0;
    }
}
